--------------------------------------------------------------------------
-- f-cpu/vhdl/eu_rop2/rop2_xbar.vhdl - ROP2 signal conditioning for the F-CPU
-- Copyright (C) 2000-2001 Yann GUIDON (whygee@f-cpu.org)
--
-- v0.2: Michael Riepe reorganized the main for-generate loop
-- + corrected the lookup table (wrong op for ORN)
-- v0.3: YG replaced UMAX/8 with MAXSIZE :-)
-- v0.4: 11/17/2000, YG wants to rewrite the unit with MR's gate library ...
-- v0.5: 8/12/2001, YG modifies the interface, the names, adds MUX,...
-- v0.6: 8/28/2001 : thanks to MR ideas, this file should now be stable.
-- version : Tue Aug 21 11:28:52 2001 by whygee@f-cpu.org
-- I just created the new entity, separating its function from the ROP2 EU.
-- version : Tue Aug 28 07:21:56 2001 by whygee@f-cpu.org :
-- the type casting problem is now avoided thanks to the m4 radix evaluation
-- capabilities that perform the modifications into f-cpu_config.vhdl.in !
-- version : jeu nov 29 20:15:19 2001 by whygee@f-cpu.org :
-- off-by-one corrected in the interface vector size.
-- 
--------------------------BEGIN-VHDL-LICENSE-----------------------------
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
---------------------------END-VHDL-LICENSE------------------------------
--
-- Warning : this code is and should remain purely combinatorial,
-- there is no latching here, it must be done at another level.
-- The function lookup table (below) is now moved earlier
-- in the pipeline, in parallel with the Xbar cycle.
-- The big fanout problems (propagation of the opcode from 1 to 64 bits)
-- overlaps the Xbar cycle so we can make a nice "signal tree".
--------------------------------------------------------------------------

LIBRARY ieee;
    USE ieee.std_logic_1164.ALL;
    USE ieee.numeric_std.all;
LIBRARY work;
    USE work.FCPU_config.ALL;
    USE work.fanout;

Entity EU_ROP2_XBAR is
  port(
    ROP2_function : in Std_ulogic_vector(2 downto 0); -- the input, "compressed" function
    ROP2_function_bit0,
    ROP2_function_bit1,   -- pre-buffered boolean function bits
    ROP2_function_bit2,
    ROP2_function_bit3 : out Std_ulogic_vector((MAXSIZE *2)-1 downto 0) -- fanout=4 
  );
end EU_ROP2_XBAR;

Architecture arch1 of EU_ROP2_XBAR is
  signal local_function : std_ulogic_vector(3 downto 0); -- the partial results.
  signal
    tree_bit0,
    tree_bit1,
    tree_bit2,
    tree_bit3 : std_ulogic_vector((MAXSIZE *2) downto 0); -- a trick for a signal tree.

begin

---- lookup table that decodes the function bits
--   with ROP_function select
--     local_function <=
--       "0001" when "000",  -- AND
--       "0010" when "001",  -- ANDN
--       "0110" when "010",  -- XOR
--       "0111" when "011",  -- OR
--       "1000" when "100",  -- NOR
--       "1001" when "101",  -- XNOR
--       "1011" when "110",  -- ORN   this one should be ok
--       "1110" when others; -- NAND

  with ROP2_function select
    local_function <=
      ROP2_VALUE_AND  when ROP2_AND,
      ROP2_VALUE_ANDN when ROP2_ANDN,
      ROP2_VALUE_XOR  when ROP2_XOR,
      ROP2_VALUE_OR   when ROP2_OR,
      ROP2_VALUE_NOR  when ROP2_NOR,
      ROP2_VALUE_XNOR when ROP2_XNOR,
      ROP2_VALUE_ORN  when ROP2_ORN,
      ROP2_VALUE_NAND when others;

-- The signal trees :
  tree_0: entity fanout
    generic map (log2_width => LOGMAXSIZE+1)
    port map (root => local_function(0), leaf => ROP2_function_bit0);
  tree_1: entity fanout
    generic map (log2_width => LOGMAXSIZE+1)
    port map (root => local_function(1), leaf => ROP2_function_bit1);
  tree_2: entity fanout
    generic map (log2_width => LOGMAXSIZE+1)
    port map (root => local_function(2), leaf => ROP2_function_bit2);
  tree_3: entity fanout
    generic map (log2_width => LOGMAXSIZE+1)
    port map (root => local_function(3), leaf => ROP2_function_bit3);

end;
